#!/usr/bin/env python2

# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import random
import sys
import traceback

sys.path.append(".")

import acos_client

instances = {
    # '2.7.2': {
    #     'host': '10.10.100.20',
    #     'port': 8443,
    #     'protocol': 'https',
    #     'user': 'admin',
    #     'password': 'a10',
    #     'axapi': '21',
    # '4.0.1': {
    #    'host': '10.48.7.77',
    #    'port': 443,
    #    'protocol': 'https',
    #    'user': 'admin',
    #    'password': 'a10',
    #    'axapi': '30',
    # },
    # '4.0.3-P1': {
    #     'host': '10.48.7.78',
    #     'port': 443,
    #     'protocol': 'https',
    #     'user': 'admin',
    #     'password': 'a10',
    #     'axapi': '30',
    # },
    # '4.0.3-GA': {
    #     'host': '10.48.7.80',
    #     'port': 443,
    #     'protocol': 'https',
    #     'user': 'admin',
    #     'password': 'a10',
    #     'axapi': '30',
    # },
    '4.0.1-GA-SP11': {
        'host': '10.48.7.79',
        'port': 443,
        'protocol': 'https',
        'user': 'admin',
        'password': 'a10',
        'axapi': '30',
    },
    # '4.1.0': {
    #     'host': '10.48.7.74',
    #     'port': 443,
    #     'protocol': 'https',
    #     'user': 'admin',
    #     'password': 'a10',
    #     'axapi': '30',
    # },
}

partitions = [
    {
        'name': 'shared',
        's1': '192.168.2.254',
        'vip1': '192.168.2.250',
        'vip2': '192.168.2.249',
        'vip3': '192.168.2.248'
    },
    {
        'name': 'p1',
        's1': '192.168.2.244',
        'vip1': '192.168.2.240',
        'vip2': '192.168.2.239',
        'vip3': '192.168.2.238'
    },
    {
        'name': 'p2',
        's1': '192.168.2.234',
        'vip1': '192.168.2.230',
        'vip2': '192.168.2.229',
        'vip3': '192.168.2.228'
    },
]


class Nope(Exception):
    pass


def get_client(h, password=None):
    p = password or h['password']
    c = acos_client.Client(h['host'], h['axapi'], h['user'], p,
                           port=h['port'],
                           protocol=h['protocol'])
    return c


def run_all(version, ax, partition, pmap):
    print("=============================================================")
    print("=============================================================")
    print("=============================================================")
    print("=============================================================")
    print("RUNNING WITH ACOS VERSION ", version)

    print("=============================================================")
    print("")
    print("Basic login/get info twice")
    c = get_client(ax)
    r = c.system.information()
    print("LIBRARY RESPONSE = %s", r)

    print("=============================================================")
    print("")
    print("About to do an authenticate with a bad password")
    try:
        c = get_client(ax, password='badpass')
        c.system.information()
    except acos_client.errors.AuthenticationFailure:
        print("got bad auth exception, good")
    else:
        sys.stdout.flush()
        raise Nope()

    print("=============================================================")
    print("")
    print("About to do a good close")
    c = get_client(ax)
    c.system.information()
    c.session.close()

    print("=============================================================")
    print("")
    print("About to do a close with no session id")
    c = get_client(ax)
    c.session.close()
    if c.session.session_id is not None:
        print("ERROR: session id is not none")
        print("ERROR: ", c.session.session_id)
        raise Nope()
    print("here comes the close with no id...")
    c.session.close()

    print("=============================================================")
    print("")
    print("About to do a close with bad session id")
    c.session.session_id = 'bad_session_id'
    c.session.close()

    # print("=============================================================")
    # print("")
    # print("About to do a get info with bad session id")
    # c.session.session_id = 'bad_session_id'
    # try:
    #     c.system.information()
    # except acos_client.errors.InvalidSessionID:
    #     print("got invalid session error, good")
    # else:
    #     raise Nope()

    # Get a fresh client

    c = get_client(ax)
    c.system.information()

    # c.ha.sync('172.18.61.27', 'admin', 'a10')

    print("=============================================================")
    print("")
    print(" PARTITIONS!!! ")

    print("=============================================================")
    print("")
    print("About to search for partition")

    p_exists = c.system.partition.exists(partition)
    print("LIBRARY RESPONSE = %s", p_exists)

    print("=============================================================")
    print("")
    print("About to make partition active (not exist, if not shared)")

    try:
        c.system.partition.active(partition)
    except acos_client.errors.NotFound:
        pass

    print("=============================================================")
    print("")
    print("About to create partition")

    if not p_exists:
        c.system.partition.create(partition)

    try:
        c.system.partition.create(partition)
    except acos_client.errors.Exists:
        pass

    print("=============================================================")
    print("")
    print("About to make partition active")

    c.system.partition.active(partition)

    print("=============================================================")
    print("")
    print("Write mem")
    c = get_client(ax)
    print("A")
    c.system.action.write_memory()

    # print("=============================================================")
    # print("")
    # print("Client SSL Create")
    # c.slb.template.client_ssl.delete("ss1")
    # c.slb.template.client_ssl.create("ss1", "cert1", "cert1")
    # c.slb.template.client_ssl.get("ss1")
    # try:
    #     c.slb.template.client_ssl.create("ss1", "cert1", "cert1")
    # except acos_client.errors.Exists:
    #     print("got already exists error, good")
    # c.slb.template.client_ssl.update("ss1", "cert1", "cert1")
    # try:
    #     c.slb.template.client_ssl.update("sns1", "cert1", "cert1")

    # except acos_client.errors.NotFound:
    #     print("got not found, good")
    # c.slb.template.client_ssl.delete("ss1")
    # c.slb.template.client_ssl.delete("ss1")
    # try:
    #     c.slb.template.client_ssl.get("ss1")
    # except acos_client.errors.NotFound:
    #     print("got not found, good")
    # c.slb.template.client_ssl.create("ss1", "cert1", "cert1")

    # print("=============================================================")
    # print("")
    # print("Server SSL Create")
    # c.slb.template.server_ssl.delete("ss1")
    # c.slb.template.server_ssl.create("ss1", "cert1", "cert1")
    # c.slb.template.server_ssl.get("ss1")
    # try:
    #     c.slb.template.server_ssl.create("ss1", "cert1", "cert1")
    # except acos_client.errors.Exists:
    #     print("got already exists error, good")
    # c.slb.template.server_ssl.update("ss1", "cert1", "cert1")
    # try:
    #     c.slb.template.server_ssl.update("sns1", "cert1", "cert1")
    # except acos_client.errors.NotFound:
    #     print("got not found, good")
    # c.slb.template.server_ssl.delete("ss1")
    # c.slb.template.server_ssl.delete("ss1")
    # try:
    #     c.slb.template.server_ssl.get("ss1")
    # except acos_client.errors.NotFound:
    #     print("got not found, good")
    # c.slb.template.server_ssl.create("ss1", "cert1", "cert1")

    print("=============================================================")
    print("")
    print("Server Create")
    c.slb.server.delete("foobar")
    c.slb.server.create("foobar", pmap['s1'])
    r = c.slb.server.get("foobar")
    print("LIBRARY RESPONSE = %s", r)
    try:
        c.slb.server.create("foobar", pmap['s1'])
    except acos_client.errors.Exists:
        print("got already exists error, good")
    else:
        raise Nope()
    c.slb.server.delete("foobar")
    c.slb.server.delete("foobar")
    try:
        c.slb.server.get("foobar")
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()
    c.slb.server.create("foobar", pmap['s1'])

    print("=============================================================")
    print("")
    print("SG Create")
    # temp -- odd that we have to delete this vport
    c.slb.virtual_server.vport.delete(
        "vip3", "vip3_VPORT", c.slb.virtual_server.vport.HTTP, 80)
    # temp -- odd that we have to delete this vport
    c.slb.service_group.delete("pfoobar")
    c.slb.service_group.create("pfoobar", c.slb.service_group.TCP,
                               c.slb.service_group.ROUND_ROBIN)
    r = c.slb.service_group.get("pfoobar")
    print("LIBRARY RESPONSE = %s", r)
    try:
        c.slb.service_group.create("pfoobar", c.slb.service_group.TCP,
                                   c.slb.service_group.ROUND_ROBIN)
    except acos_client.errors.Exists:
        print("got already exists error, good")
    else:
        raise Nope()
    c.slb.service_group.update("pfoobar", c.slb.service_group.TCP,
                               c.slb.service_group.LEAST_CONNECTION)
    try:
        c.slb.service_group.update("pnfoobar", c.slb.service_group.TCP,
                                   c.slb.service_group.LEAST_CONNECTION)
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()
    c.slb.service_group.delete("pfoobar")
    c.slb.service_group.delete("pfoobar")
    try:
        c.slb.service_group.get("pfoobar")
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()
    c.slb.service_group.create("pfoobar", c.slb.service_group.TCP,
                               c.slb.service_group.ROUND_ROBIN)

    print("=============================================================")
    print("")
    print("VIP Create")
    c.slb.virtual_server.delete("vip3")
    c.slb.virtual_server.create("vip3", pmap['vip3'])
    c.slb.virtual_server.get("vip3")

    c.slb.virtual_server.delete("vfoobar")
    c.slb.virtual_server.create("vfoobar", pmap['vip1'])
    r = c.slb.virtual_server.get("vfoobar")
    print("LIBRARY RESPONSE = %s", r)
    r = c.slb.virtual_server.all()
    print("LIBRARY RESPONSE = %s", r)
    try:
        c.slb.virtual_server.create("vfoobar", pmap['vip1'])
    except acos_client.errors.Exists:
        print("got already exists error, good")
    else:
        raise Nope()
    try:
        c.slb.virtual_server.stats("vfoobar")
    except Exception:
        pass
    c.slb.virtual_server.delete("vfoobar")
    c.slb.virtual_server.delete("vfoobar")
    try:
        c.slb.virtual_server.get("vfoobar")
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()

    c.slb.virtual_server.vport.delete("vip3", "vip3_VPORT",
                                      c.slb.virtual_server.vport.HTTP, 80)
    c.slb.virtual_server.vport.create("vip3", "vip3_VPORT",
                                      service_group_name="pfoobar",
                                      protocol=c.slb.virtual_server.vport.HTTP,
                                      port='80')
    try:
        c.slb.virtual_server.vport.create(
            "vip3", "vip3_VPORT",
            service_group_name="pfoobar",
            protocol=c.slb.virtual_server.vport.HTTP,
            port='80')
    except acos_client.errors.Exists:
        print("got already exists error, good")
    else:
        raise Nope()
    c.slb.virtual_server.vport.delete("vip3", "vip3_VPORT",
                                      c.slb.virtual_server.vport.HTTP, 80)
    c.slb.virtual_server.vport.delete("vip3", "vip3_VPORT",
                                      c.slb.virtual_server.vport.HTTP, 80)

    print("=============================================================")
    print("")
    print("HM Create")
    c.slb.hm.delete("hnfoobar")
    c.slb.hm.delete("hfoobar")
    c.slb.hm.create("hfoobar", c.slb.hm.HTTP, 5, 5, 5, 'GET', '/', '200', 80)
    r = c.slb.hm.get("hfoobar")
    print("LIBRARY RESPONSE = %s", r)
    try:
        c.slb.hm.create("hfoobar", c.slb.hm.HTTP, 5, 5, 5, 'GET', '/', '200',
                        80)
    except acos_client.errors.Exists:
        print("got already exists error, good")
    else:
        raise Nope()
    c.slb.hm.update("hfoobar", c.slb.hm.HTTP, 10, 10, 10, 'GET', '/', '200', 80)
    try:
        c.slb.hm.update("hnfoobar", c.slb.hm.HTTP, 10, 10, 10, 'GET', '/', '200', 80)
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()
    c.slb.hm.delete("hfoobar")
    c.slb.hm.delete("hfoobar")
    try:
        c.slb.hm.get("hfoobar")
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()

    c.slb.hm.delete("hm2")
    c.slb.hm.create("hm2", c.slb.hm.ICMP, 5, 5, 5)
    r = c.slb.hm.get("hm2")

    c.slb.hm.delete("hm4")
    c.slb.hm.create("hm4", c.slb.hm.TCP, 5, 5, 5, port=25)
    r = c.slb.hm.get("hm4")

    c.slb.hm.delete("hm3")
    c.slb.hm.create("hm3", c.slb.hm.HTTPS, 5, 5, 5, 'GET', '/', '200', 443)
    r = c.slb.hm.get("hm3")

    print("=============================================================")
    print("")
    print("Member Create")
    c.slb.service_group.member.delete("pfoobar", "foobar", 80)
    c.slb.service_group.member.create("pfoobar", "foobar", 80)
    try:
        c.slb.service_group.member.create("pfoobar", "foobar", 80)
    except acos_client.errors.Exists:
        print("got already exists error, good")
    else:
        raise Nope()
    c.slb.service_group.member.update("pfoobar", "foobar", 80,
                                      c.slb.DOWN)
    try:
        c.slb.service_group.member.update("pfoobar", "nfoobar", 80)
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()
    try:
        c.slb.service_group.member.update("pnfoobar", "foobar", 80)
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()
    c.slb.service_group.member.delete("pfoobar", "foobar", 80)
    c.slb.service_group.member.delete("pfoobar", "foobar", 80)

    print("=============================================================")
    print("")
    print("Source Ip Persistence")
    c.slb.template.src_ip_persistence.delete("sip1")
    c.slb.template.src_ip_persistence.create("sip1")
    try:
        c.slb.template.src_ip_persistence.create("sip1")
    except acos_client.errors.Exists:
        print("got already exists error, good")
    else:
        raise Nope()
    r = c.slb.template.src_ip_persistence.get("sip1")
    print("LIBRARY RESPONSE = %s", r)
    c.slb.template.src_ip_persistence.exists("sip1")
    c.slb.template.src_ip_persistence.delete("sip1")
    c.slb.template.src_ip_persistence.delete("sip1")
    try:
        c.slb.template.src_ip_persistence.get("sip1")
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()
    c.slb.template.src_ip_persistence.create("sip1")

    print("=============================================================")
    print("")
    print("Http Cookie Persistence")
    c.slb.template.cookie_persistence.delete("cp1")
    c.slb.template.cookie_persistence.create("cp1")
    try:
        c.slb.template.cookie_persistence.create("cp1")
    except acos_client.errors.Exists:
        print("got already exists error, good")
    else:
        raise Nope()
    c.slb.template.cookie_persistence.get("cp1")
    c.slb.template.cookie_persistence.exists("cp1")
    c.slb.template.cookie_persistence.delete("cp1")
    c.slb.template.cookie_persistence.delete("cp1")
    try:
        c.slb.template.cookie_persistence.get("cp1")
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()
    c.slb.template.cookie_persistence.create("cp1")

    print("=============================================================")
    print("")
    print("Vip with pers")
    c.slb.virtual_server.delete("vip2")
    c.slb.virtual_server.create("vip2", pmap['vip2'])
    c.slb.virtual_server.vport.create(
        "vip2", "vip2_vport2",
        protocol=c.slb.virtual_server.vport.HTTPS,
        port=444,
        service_group_name='pfoobar',
        s_pers_name='sip1')
    c.slb.virtual_server.vport.create(
        "vip2", "vip2_vport3",
        protocol=c.slb.virtual_server.vport.HTTPS,
        port=445,
        service_group_name='pfoobar',
        c_pers_name='cp1')

    print("=============================================================")
    print("")
    print("About half the time, delete the partition!")

    if int(random.random() * 2):
        c.system.partition.delete(partition)
        try:
            c.system.partition.delete(partition)
        except acos_client.errors.NotFound:
            pass

    c.session.close()


def main():
    # for v in partitions:
    #     partition = v['name']
    #     for version, ax in instances.items():
    #         try:
    #             run_all(version, ax, partition, v)
    #         except Exception as e:
    #             traceback.print_exc()
    #             print(e)
    #             sys.exit(1)
    for version, ax in instances.items():
        try:
            run_all(version, ax, 'shared', partitions[0])
        except Exception as e:
            traceback.print_exc()
            print(e)
            sys.exit(1)

if __name__ == '__main__':
    main()
    sys.exit(0)



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.v30.base as base


class VirtualPort(base.BaseV30):

    # Protocols
    TCP = "tcp"
    UDP = "udp"
    OTHERS = "others"
    DIAMETER = "diameter"
    DNS_TCP = "dns-tcp"
    DNS_UDP = "dns-udp"
    FAST_HTTP = "fast-http"
    FIX = "fix"
    FTP = "ftp"
    FTP_PROXY = "ftp-proxy"
    HTTP = "http"
    HTTPS = "https"
    MLB = "mlb"
    MMS = "mms"
    MYSQL = "mysql"
    MSSQL = "mssql"
    RADIUS = "radius"
    RTSP = "rtsp"
    SIP = "sip"
    SIP_TCP = "sip-tcp"
    SIPS = "sips"
    SMPP_TCP = "smpp-tcp"
    SPDY = "spdy"
    SPDYS = "spdys"
    SMTP = "smtp"
    SSL_PROXY = "ssl-proxy"
    TCP_PROXY = "tcp-proxy"
    TFTP = "tftp"
    GENERIC_PROXY = "tcp-proxy"

    url_server_tmpl = '/slb/virtual-server/{name}/port/'
    url_port_tmpl = '{port_number}+{protocol}'

    def all(self, virtual_server_name, **kwargs):
        url = self.url_server_tmpl.format(name=virtual_server_name)
        return self._get(url, **kwargs)

    def _set(self, virtual_server_name, name, protocol, port,
             service_group_name,
             s_pers_name=None, c_pers_name=None, stats=0, update=False,
             **kwargs):

        params = {
            "port": self.minimal_dict({
                "name": name,
                "service-group": service_group_name,
                "protocol": protocol,
                "port-number": int(port),
                "template-persist-source-ip": s_pers_name,
                "template-persist-cookie": c_pers_name,
                "extended-stats": stats
            }, exclude=['template-persist-source-ip', 'template-persist-cookie'])
        }

        url = self.url_server_tmpl.format(name=virtual_server_name)
        if update:
            url += self.url_port_tmpl.format(
                port_number=port, protocol=protocol
            )

        return self._post(url, params, **kwargs)

    def create(self, virtual_server_name, name, protocol, port,
               service_group_name,
               s_pers_name=None, c_pers_name=None, status=1, **kwargs):
        return self._set(virtual_server_name,
                         name, protocol, port, service_group_name,
                         s_pers_name, c_pers_name, status, **kwargs)

    def update(self, virtual_server_name, name, protocol, port,
               service_group_name,
               s_pers_name=None, c_pers_name=None, status=1, **kwargs):
        return self._set(virtual_server_name,
                         name, protocol, port, service_group_name,
                         s_pers_name, c_pers_name, status, True, **kwargs)

    def delete(self, virtual_server_name, name, protocol, port):
        url = self.url_server_tmpl.format(name=virtual_server_name)
        url += self.url_port_tmpl.format(port_number=port, protocol=protocol)
        return self._delete(url)



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.errors as acos_errors
import acos_client.v30.base as base


class Server(base.BaseV30):

    url_prefix = '/slb/server/'

    def get(self, name, **kwargs):
        return self._get(self.url_prefix + name, **kwargs)

    def create(self, name, ip_address, **kwargs):
        params = {
            "server": {
                "name": name,
                "host": ip_address,
            }
        }

        # Two creates in a row apparently works in ACOS 4.0; stop that
        try:
            self.get(name, **kwargs)
        except acos_errors.NotFound:
            pass
        else:
            raise acos_errors.Exists()

        return self._post(self.url_prefix, params, **kwargs)

    def delete(self, name):
        return self._delete(self.url_prefix + name)



# Copyright 2016, All Rights Reserved,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

CLEAN_FIELDS = ["username", "password"]

REPLACEMENT = "*" * 8
PRIMITIVES = [int, float, str]


def clean(data):
    data = copy.copy(data)

    if type(data) is dict:
        for x, y in data.iteritems():
            if type(y) is dict:
                data[x] = clean(y)
            else:
                if x in CLEAN_FIELDS:
                    data[x] = REPLACEMENT
    elif hasattr(data, "__dict__"):
        for x, y in data.__dict__.iteritems():
            if type(y) is dict or hasattr(y, "__dict__"):
                setattr(data, x, clean(y))
            else:
                if x in CLEAN_FIELDS:
                    setattr(data, x, REPLACEMENT)
    return data



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import unittest2 as unittest
import v21_mocks as mocks

import acos_client


class TestSession(unittest.TestCase):

    def test_id(self):
        s = mocks.Session({'username': 'mongo', 'password': 'teddybear',
                           'session_id': 'fruitloops'})
        with s.client() as c:
            self.assertEqual(c.session.session_id, None)
            self.assertEqual(c.session.id, 'fruitloops')
            self.assertEqual(c.session.session_id, 'fruitloops')

    def test_authenticate(self):
        s = mocks.Session({'username': 'panda', 'password': 'weak'})
        with s.client() as c:
            r = c.session.authenticate('panda', 'weak')
            self.assertNotEqual(r['session_id'], None)

    def test_bad_authenticate(self):
        s = mocks.SessionBadPassword({'username': 'user', 'password': 'pass'})
        with s.client() as c:
            self.assertEqual(c.session.session_id, None)
            try:
                c.session.authenticate('user', 'pass')
            except acos_client.errors.AuthenticationFailure:
                pass
            self.assertEqual(c.session.session_id, None)

    def test_close(self):
        m = mocks.Close()
        with m.client() as c:
            c.session.close()
            self.assertEqual(c.session.session_id, None)

    def test_close_with_bad_session_id(self):
        m = mocks.CloseBadSession({'session_id': 'badsessionid'})
        with m.client() as c:
            c.session.close()
            self.assertEqual(c.session.session_id, None)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.v21.base as base


class Port(base.BaseV21):

    # Protocols
    TCP = 2
    UDP = 3

    def _set(self, action, name, port_num, protocol, **kwargs):

        params = {
            "name": name,
            "port": {
                "port_num": port_num,
                "protocol": protocol,
            }
        }

        return self._post(action, params, **kwargs)

    def create(self, name, port_num, protocol, **kwargs):
        return self._set("slb.server.port.create", name, port_num, protocol,
                         **kwargs)

    def update(self, name, port_num, protocol, **kwargs):
        return self._set("slb.server.port.update", name, port_num, protocol,
                         **kwargs)

    def all_update(self, name, port_num, protocol, **kwargs):
        return self._set("slb.server.port.updateAll", name, port_num, protocol,
                         **kwargs)

    def delete(self, name, port_num, protocol, **kwargs):
        self._set("slb.server.port.delete", name, port_num, protocol, **kwargs)

    def all_delete(self, name, **kwargs):
        self._get("slb.server.port.deleteAll",  {"name": name}, **kwargs)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client

import errors as acos_errors
import v21.axapi_http
from v21.ha import HA as v21_HA
from v21.nat import Nat as v21_Nat
from v21.network import Network as v21_Network
from v21.session import Session as v21_Session
from v21.slb import SLB as v21_SLB
from v21.system import System as v21_System
import v30.axapi_http
from v30.file import File as v30_File
from v30.ha import HA as v30_HA
from v30.nat import Nat as v30_Nat
from v30.network import Network as v30_Network
from v30.session import Session as v30_Session
from v30.slb import SLB as v30_SLB
from v30.system import System as v30_System

VERSION_IMPORTS = {
    '21': {
        'http': v21.axapi_http,
        'HA': v21_HA,
        'Nat': v21_Nat,
        'Network': v21_Network,
        'Session': v21_Session,
        'SLB': v21_SLB,
        'System': v21_System,
    },
    '30': {
        'http': v30.axapi_http,
        'HA': v30_HA,
        'Nat': v30_Nat,
        'Network': v30_Network,
        'Session': v30_Session,
        'SLB': v30_SLB,
        'System': v30_System,
        'File': v30_File
    },
}


class Client(object):

    def __init__(self, host, version, username, password, port=None,
                 protocol="https", timeout=None):
        self._version = self._just_digits(version)
        if self._version not in acos_client.AXAPI_VERSIONS:
            raise acos_errors.ACOSUnsupportedVersion()
        self.http = VERSION_IMPORTS[self._version]['http'].HttpClient(
            host, port, protocol, timeout=timeout)
        self.session = VERSION_IMPORTS[self._version]['Session'](
            self, username, password)
        self.current_partition = 'shared'

    def _just_digits(self, s):
        return ''.join(i for i in str(s) if i.isdigit())

    @property
    def ha(self):
        return VERSION_IMPORTS[self._version]['HA'](self)

    @property
    def system(self):
        return VERSION_IMPORTS[self._version]['System'](self)

    @property
    def slb(self):
        return VERSION_IMPORTS[self._version]['SLB'](self)

    @property
    def network(self):
        return VERSION_IMPORTS[self._version]['Network'](self)

    @property
    def nat(self):
        return VERSION_IMPORTS[self._version]['Nat'](self)

    @property
    def file(self):
        return VERSION_IMPORTS[self._version]['File'](self)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import errno
import httplib
import json
import logging
import socket
import ssl
import sys
import time
import urlparse

import responses as acos_responses

import acos_client
from acos_client import logutils

LOG = logging.getLogger(__name__)

out_hdlr = logging.StreamHandler(sys.stderr)
out_hdlr.setLevel(logging.ERROR)
LOG.addHandler(out_hdlr)
LOG.setLevel(logging.ERROR)


# Monkey patch for ssl connect, for specific TLS version required by
# ACOS hardware.
def force_tlsv1_connect(self):
    sock = socket.create_connection((self.host, self.port), self.timeout)
    if self._tunnel_host:
        self.sock = sock
        self._tunnel()
    self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file,
                                ssl_version=ssl.PROTOCOL_TLSv1)


def extract_method(api_url):
    q = urlparse.urlparse(api_url).query
    return urlparse.parse_qs(q).get('method', [u''])[0]


def merge_dicts(d1, d2):
    d = d1.copy()
    for k, v in d2.items():
        if k in d and isinstance(d[k], dict):
            d[k] = merge_dicts(d[k], d2[k])
        else:
            d[k] = d2[k]
    return d


broken_replies = {
    ('<?xml version="1.0" encoding="utf-8" ?><response status="ok">'
     '</response>'): (json.dumps({"response": {"status": "OK"}})),

    ('<?xml version="1.0" encoding="utf-8" ?><response status="fail">'
     '<error code="999" msg=" Partition does not exist. '
     '(internal error: 520749062)" /></response>'):
    (json.dumps({"response": {"status": "fail", "err": {"code": 999,
     "msg": " Partition does not exist."}}})),

    ('<?xml version="1.0" encoding="utf-8" ?><response status="fail">'
     '<error code="999" msg=" Failed to get partition. (internal error: '
     '402718800)" /></response>'):
    (json.dumps({"response": {"status": "fail", "err": {"code": 999,
     "msg": " Partition does not exist."}}})),

    ('<?xml version="1.0" encoding="utf-8" ?><response status="fail">'
     '<error code="1076" msg="Invalid partition parameter." /></response>'):
    (json.dumps({"response": {"status": "fail", "err": {"code": 1076,
     "msg": "Invalid partition parameter."}}})),

    ('<?xml version="1.0" encoding="utf-8" ?><response status="fail">'
     '<error code="999" msg=" No such aFleX. (internal error: '
     '17039361)" /></response>'):
    (json.dumps({"response": {"status": "fail", "err": {"code": 17039361,
     "msg": " No such aFleX."}}})),

    ('<?xml version="1.0" encoding="utf-8" ?><response status="fail">'
     '<error code="999" msg=" This aFleX is in use. (internal error: '
     '17039364)" /></response>'):
    (json.dumps({"response": {"status": "fail", "err": {"code": 17039364,
     "msg": " This aFleX is in use."}}})),

}


class EmptyHttpResponse(Exception):
    def __init__(self, response):
        self.response = response


class HttpClient(object):
    HEADERS = {
        "Content-type": "application/json",
        "User-Agent": "ACOS-Client-AGENT-%s" % acos_client.VERSION,
    }

    headers = {}

    def __init__(self, host, port=None, protocol="https", client=None, timeout=None):
        self.host = host
        self.port = port
        self.protocol = protocol
        self.timeout = timeout
        if port is None:
            if protocol is 'http':
                self.port = 80
            else:
                self.port = 443

    def _http(self, method, api_url, payload):
        if self.protocol == 'https':
            http = httplib.HTTPSConnection(self.host, self.port, timeout=self.timeout)
            http.connect = lambda: force_tlsv1_connect(http)
        else:
            http = httplib.HTTPConnection(self.host, self.port, timeout=self.timeout)

        LOG.debug("axapi_http: url:     %s", api_url)
        LOG.debug("axapi_http: method:  %s", method)
        LOG.debug("axapi_http: headers: %s", logutils.clean(self.HEADERS))
        LOG.debug("axapi_http: payload: %s", logutils.clean(payload))

        http.request(method, api_url, body=payload, headers=self.headers)

        r = http.getresponse()

        # Workaround for zero length response
        def handle_empty_response(data):
            if not data:
                raise EmptyHttpResponse(r)

            return data

        return handle_empty_response(r.read())

    def request(self, method, api_url, params={}, **kwargs):
        LOG.debug("axapi_http: url = %s", api_url)
        LOG.debug("axapi_http: params = %s", logutils.clean(params))

        self.headers = self.HEADERS

        if params:
            extra_params = kwargs.get('axapi_args', {})
            params_copy = merge_dicts(params, extra_params)
            LOG.debug("axapi_http: params_all = %s", logutils.clean(params_copy))

            payload = json.dumps(params_copy, encoding='utf-8')
        else:
            try:
                payload = kwargs.pop('payload', None)
                self.headers = dict(self.headers, **kwargs.pop('headers', {}))
                LOG.debug("axapi_http: headers_all = %s", logutils.clean(self.headers))
            except KeyError:
                payload = None

        last_e = None

        for i in xrange(0, 600):
            try:
                last_e = None
                data = self._http(method, api_url, payload)
                break
            except socket.error as e:
                # Workaround some bogosity in the API
                if (e.errno == errno.ECONNRESET or
                   e.errno == errno.ECONNREFUSED):
                    time.sleep(0.1)
                    last_e = e
                    continue
                raise e
            except httplib.BadStatusLine as e:
                time.sleep(0.1)
                last_e = e
                continue
            except EmptyHttpResponse as e:
                if e.response.status != httplib.OK:
                    msg = dict(e.response.msg.items())
                    data = json.dumps({"response": {'status': 'fail', 'err':
                                      {'code': e.response.status,
                                       'msg': msg}}})
                else:
                    data = json.dumps({"response": {"status": "OK"}})
                break

        if last_e is not None:
            raise e

        LOG.debug("axapi_http: data = %s", logutils.clean(data))

        # Fixup some broken stuff in an earlier version of the axapi
        # xmlok = ('<?xml version="1.0" encoding="utf-8" ?>'
        #          '<response status="ok"></response>')
        # if data == xmlok:
        #     return {'response': {'status': 'OK'}}
        # TODO()
        if data in broken_replies:
            data = broken_replies[data]
            LOG.debug("axapi_http: broken reply, new response: %s",
                      logutils.clean(data))

        try:
            r = json.loads(data, encoding='utf-8')
        except ValueError as e:
            # Handle non json response
            LOG.debug("axapi_http: json = %s", e)
            return data

        if 'response' in r and 'status' in r['response']:
            if r['response']['status'] == 'fail':
                    acos_responses.raise_axapi_ex(
                        r, action=extract_method(api_url))

        return r

    def get(self, api_url, params={}, **kwargs):
        return self.request("GET", api_url, params, **kwargs)

    def post(self, api_url, params={}, **kwargs):
        return self.request("POST", api_url, params, **kwargs)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import unittest2 as unittest
import v21_mocks as mocks


class TestSystem(unittest.TestCase):

    def test_information(self):
        m = mocks.SystemInformation()
        with m.client() as c:
            r = c.system.information()
            self.assertIn('system_information', r)

    def test_write_memory(self):
        m = mocks.SystemWriteMemory()
        with m.client() as c:
            c.system.action.write_memory()



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.errors as acos_errors

import base


class Action(base.BaseV21):

    def write_memory(self):
        try:
            self._get("system.action.write_memory")
        except acos_errors.InvalidPartitionParameter:
            pass

    def reboot(self, **kwargs):
        raise NotImplementedError
        # return self._post("system.action.reboot", **kwargs)

    def reload(self, write_memory=False, **kwargs):
        # write_memory param is required but no matter what value is passed
        # it will ALWAYS save pending changes
        write_memory = 1 if write_memory else 0
        return self._post("system.action.reload",
                          params={"write_memory": write_memory}, **kwargs)

    def activate_and_write(self, partition, **kwargs):
        write_format = "active-partition {0}\r\nwrite memory\r\n"
        post_body = write_format.format(partition)

        return self._request("POST", "cli.deploy", params=None, payload=post_body, **kwargs)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.v21.base as base


class VirtualService(base.BaseV21):

    def all(self, **kwargs):
        return self._get("slb.virtual_service.getAll", **kwargs)

    def get(self, name, **kwargs):
        return self._post("slb.virtual_service.search", {'name': name},
                          **kwargs)

    def _set(self, action, name, protocol, port, **kwargs):
        params = {
            "virtual_service": {
                "port": port,
                "protocol": protocol,
                "name": name
            }
        }

        return self._post(action, params, **kwargs)

    def create(self, name, protocol, port, **kwargs):
        return self._set("slb.virtual_service.create", name, protocol, port,
                         **kwargs)

    def update(self, name, protocol, port, **kwargs):
        return self._set("slb.virtual_service.update", name, protocol, port,
                         **kwargs)

    def delete(self, name, **kwargs):
        return self._post("slb.virtual_service.delete", {"name": name},
                          **kwargs)

    def all_delete(self, **kwargs):
        return self._get("slb.virtual_service.deleteAll", **kwargs)

    def stats(self, name, **kwargs):
        return self._post("slb.virtual_service.fetchStatistics",
                          {"name": name}, **kwargs)

    def all_stats(self, **kwargs):
        return self._get("slb.virtual_service.fetchAllStatistics", **kwargs)



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.errors as acos_errors
import acos_client.v30.base as base

from member import Member


class ServiceGroup(base.BaseV30):
    url_prefix = '/slb/service-group/'

    @property
    def member(self):
        return Member(self.client)

    # Valid LB methods
    ROUND_ROBIN = 'round-robin'
    WEIGHTED_ROUND_ROBIN = 'weighted-rr'
    LEAST_CONNECTION = 'least-connection'
    WEIGHTED_LEAST_CONNECTION = 'weighted-least-connection'
    LEAST_CONNECTION_ON_SERVICE_PORT = 'service-least-connection'
    WEIGHTED_LEAST_CONNECTION_ON_SERVICE_PORT = \
        'service-weighted-least-connection'
    FAST_RESPONSE_TIME = 'fastest-response'
    LEAST_REQUEST = 'least-request'
    STRICT_ROUND_ROBIN = 'round-robin-strict'
    STATELESS_SOURCE_IP_HASH = 'stateless-src-ip-hash'
    STATELESS_SOURCE_IP_HASH_ONLY = 'stateless-src-ip-only-hash'
    STATELESS_DESTINATION_IP_HASH = 'stateless-dst-ip-hash'
    STATELESS_SOURCE_DESTINATION_IP_HASH = 'stateless-src-dst-ip-hash'
    STATELESS_PER_PACKET_ROUND_ROBIN = 'stateless-per-pkt-round-robin'

    # Valid protocols
    TCP = 'tcp'
    UDP = 'udp'

    def get(self, name, **kwargs):
        return self._get(self.url_prefix + name, **kwargs)

    def _set(self, name, protocol=None, lb_method=None, hm_name=None,
             update=False, **kwargs):

        # Normalize "" -> None for json
        hm_name = hm_name or None

        # v30 needs unit tests badly...

        params = {
            "service-group": self.minimal_dict({
                "name": name,
                "protocol": protocol,
            })
        }

        # If we explicitly disable health checks, ensure it happens
        # Else, we implicitly disable health checks if not specified.
        health_check_disable = 1 if kwargs.get("health_check_disable", False) else 0

        # When enabling/disabling a health monitor, you can't specify
        # health-check-disable and health-check at the same time.
        if hm_name is None:
            params["service-group"]["health-check-disable"] = health_check_disable
        else:
            params["service-group"]["health-check"] = hm_name

        if lb_method is None:
            pass
        elif lb_method[-16:] == 'least-connection':
            params['service-group']['lc-method'] = lb_method
        elif lb_method[:9] == 'stateless':
            params['service-group']['stateless-lb-method'] = lb_method
        else:
            params['service-group']['lb-method'] = lb_method

        if not update:
            name = ''

        self._post(self.url_prefix + name, params, **kwargs)

    def create(self, name, protocol=TCP, lb_method=ROUND_ROBIN, **kwargs):
        try:
            self.get(name)
        except acos_errors.NotFound:
            pass
        else:
            raise acos_errors.Exists

        self._set(name, protocol, lb_method, **kwargs)

    def update(self, name, protocol=None, lb_method=None, health_monitor=None,
               **kwargs):
        self._set(name, protocol, lb_method,
                  health_monitor, update=True, **kwargs)

    def delete(self, name):
        self._delete(self.url_prefix + name)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.v21.base as base


class Member(base.BaseV21):

    def _write(self, action, service_group_name, server_name, server_port,
               status=None, **kwargs):
        params = {
            "name": service_group_name,
            "member": self.minimal_dict({
                "server": server_name,
                "port": int(server_port),
                "status": status
            })
        }
        self._post(action, params, **kwargs)

    def create(self, service_group_name, server_name, server_port, status=1,
               **kwargs):
        self._write("slb.service_group.member.create", service_group_name,
                    server_name, server_port, status, **kwargs)

    def update(self, service_group_name, server_name, server_port, status=1,
               **kwargs):
        self._write("slb.service_group.member.update", service_group_name,
                    server_name, server_port, status, **kwargs)

    def delete(self, service_group_name, server_name, server_port, **kwargs):
        self._write("slb.service_group.member.delete", service_group_name,
                    server_name, int(server_port), **kwargs)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.errors as ae


RESPONSE_CODES = {
    999: {
        '*': ae.NotFound
    },
    1002: {
        '*': ae.MemoryFault
    },
    1009: {
        'session.close': None,
        '*': ae.InvalidSessionID
    },
    1023: {
        'slb.service_group.member.delete': None,
        '*': ae.NotFound
    },
    1043: {
        'slb.virtual_server.vport.delete': None,
        '*': ae.NotFound
    },
    1076: {
        'session.close': None,
        '*': ae.InvalidPartitionParameter
    },
    1163: {
        '*': ae.InvalidParameter
    },
    1165: {
        '*': ae.HMMissingHttpPassive
    },
    1405: {
        '*': ae.Exists
    },
    1406: {
        '*': ae.Exists
    },
    1982: {
        '*': ae.Exists
    },
    2941: {
        '*': ae.Exists
    },
    3602: {
        'slb.class_list.update': ae.NotFound,
        '*': ae.NotFound
    },
    17039361: {
        'slb.aflex.delete': None,
        '*': ae.NotFound
    },
    17039364: {
        'slb.aflex.upload': ae.InUse,
        'slb.aflex.delete': ae.InUse,
        '*': ae.InUse
    },
    33619968: {
        'slb.hm.delete': None,
        '*': ae.NotFound
    },
    33619969: {
        '*': ae.InUse,
    },
    67174402: {
        'slb.server.delete': None,
        'slb.server.port.delete': None,
        '*': ae.NotFound
    },
    67239937: {
        'slb.virtual_server.delete': None,
        'slb.virtual_service.delete': None,
        'slb.virtual_service.update': ae.NotFound,
        '*': ae.NotFound
    },
    67239947: {
        '*': ae.Exists
    },
    67305473: {
        'slb.service_group.delete': None,
        'slb.service_group.member.delete': None,
        'slb.service_group.member.create': ae.NotFound,
        'slb.service_group.member.update': ae.NotFound,
        '*': ae.NotFound
    },
    67371009: {
        'slb.template.cookie_persistence.delete': None,
        'slb.template.src_ip_persistence.delete': None,
        'slb.template.client_ssl.delete': None,
        'slb.template.server_ssl.delete': None,
        '*': ae.NotFound
    },
    67371049: {
        'slb.class_list.delete': None,
        '*': ae.NotFound
    },
    402653200: {
        '*': ae.Exists
    },
    402653201: {
        '*': ae.Exists
    },
    402653202: {
        '*': ae.Exists
    },
    402653206: {
        '*': ae.Exists
    },
    402718800: {
        '*': ae.NotFound
    },
    520486915: {
        '*': ae.AuthenticationFailure
    },
    520749062: {
        '*': ae.NotFound
    },
    654311465: {
        '*': ae.AddressSpecifiedIsInUse
    },
    654311495: {
        '*': ae.InUse,
    },
    654311496: {
        '*': ae.AddressSpecifiedIsInUse
    },
    654376968: {
        'nat.pool.delete': None,
        '*': ae.NotFound
    },
    654573574: {
        'network.acl.ext.delete': None,
        '*': ae.NotFound
    }
}


def raise_axapi_ex(response, action=None):
    if 'response' in response and 'err' in response['response']:
        code = response['response']['err']['code']

        if code in RESPONSE_CODES:
            ex_dict = RESPONSE_CODES[code]

            if action is not None and action in ex_dict:
                ex = ex_dict[action]
            else:
                ex = ex_dict.get('*')

            if ex is not None:
                raise ex(code, response['response']['err']['msg'])
            else:
                return

        raise ae.ACOSException(code, response['response']['err']['msg'])
    raise ae.ACOSException()



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.errors as acos_errors


class Session(object):

    def __init__(self, client, username, password):
        self.client = client
        self.http = client.http
        self.username = username
        self.password = password
        self.session_id = None

    @property
    def id(self):
        if self.session_id is None:
            self.authenticate(self.username, self.password)
        return self.session_id

    def authenticate(self, username, password):
        url = "/services/rest/v2.1/?format=json&method=authenticate"
        params = {
            "username": username,
            "password": password
        }

        if self.session_id is not None:
            self.close()

        r = self.http.post(url, params)
        self.session_id = r['session_id']
        return r

    def close(self):
        try:
            self.client.partition.active()
        except Exception:
            pass

        try:
            url = ("/services/rest/v2.1/?format=json&method=session"
                   ".close&session_id=%s" % self.session_id)

            r = self.http.post(url, {"session_id": self.session_id})
        except acos_errors.InvalidPartitionParameter:
            pass
        finally:
            self.session_id = None

        return r



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import base


class ConfigFile(base.BaseV21):

    def upload(self, cfg_backup, **kwargs):
        return self._post("system.config_file.upload", cfg_backup, **kwargs)

    def restore(self, **kwargs):
        return self._post("system.config_file.restore", **kwargs)

    def write(self, from_file, to_file, **kwargs):
        return self._post("system.config_file.write",
                          {"from": from_file, "to": to_file}, **kwargs)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.errors as acos_errors

import base


class Partition(base.BaseV21):

    def exists(self, name):
        if name == 'shared':
            return True
        try:
            self._post("system.partition.search", {'name': name})
            return True
        except acos_errors.NotFound:
            return False

    def active(self, name='shared'):
        if self.client.current_partition != name:
            self._post("system.partition.active", {'name': name})
            self.client.current_partition = name

    def create(self, name):
        params = {
            'partition': {
                'max_aflex_file': 32,
                'network_partition': 0,
                'name': name
            }
        }
        if name != 'shared':
            self._post("system.partition.create", params)

    def delete(self, name):
        if name != 'shared':
            self.client.session.close()
            self._post("system.partition.delete", {"name": name})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from acos_client import multipart
import acos_client.v21.base as base


class Aflex(base.BaseV21):

    def _set(self, action, name, aflex, **kwargs):
        m = multipart.Multipart()
        m.file(name="upload_aflex", filename=name, value=aflex)
        ct, payload = m.get()
        kwargs.update(payload=payload, headers={'Content-type': ct})
        return self._post(action, **kwargs)

    def upload(self, name, aflex, **kwargs):
        return self._set('slb.aflex.upload', name, aflex, **kwargs)

    def update(self, name, aflex, **kwargs):
        return self._set('slb.aflex.update', name, aflex, **kwargs)

    def all(self, **kwargs):
        return self._get('slb.aflex.getAll')

    def get(self, name, **kwargs):
        return self._post('slb.aflex.search', {'name': name},
                          **kwargs)

    def download(self, name, **kwargs):
        return self._post('slb.aflex.download', {'name': name},
                          **kwargs)

    def delete(self, name, **kwargs):
        self._post('slb.aflex.delete', {'name': name},
                   **kwargs)

    def stats(self, name, **kwargs):
        return self._post("slb.aflex.fetchStatistics", {"name": name},
                          **kwargs)

    def all_stats(self, **kwargs):
        return self._get("slb.aflex.fetchAllstatistics", **kwargs)

    def clear_stats(self, name, **kwargs):
        return self._post("slb.aflex.slb.aflex.clearStatistics",
                          {"name": name}, **kwargs)

    def clear_all_stats(self, **kwargs):
        return self._post("slb.aflex.clearAllStatistics", **kwargs)

    def clear_events(self, name, event_name, **kwargs):
        params = {
            "aflex_event": {
                "name": name,
                "event_name": event_name
            }
        }
        return self._post("slb.aflex.clearEvents", params, **kwargs)

    def clear_all_events(self, **kwargs):
        return self._post("slb.aflax.clearAllEvents", **kwargs)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


# TODO(mdurrant) - Organize these imports
import json
import logging
import sys

import requests


if sys.version_info >= (3, 0):
    import http.client as http_client
else:
    # Python 2
    import httplib as http_client
http_client.HTTPConnection.debuglevel = logging.INFO

import responses as acos_responses

import acos_client
from acos_client import logutils

LOG = logging.getLogger(__name__)

import sys
out_hdlr = logging.StreamHandler(sys.stdout)
out_hdlr.setLevel(logging.DEBUG)
LOG.addHandler(out_hdlr)

LOG.setLevel(logging.DEBUG)


broken_replies = {
    "": '{"response": {"status": "OK"}}'
}


class HttpClient(object):
    HEADERS = {
        "Content-type": "application/json",
        "User-Agent": "ACOS-Client-AGENT-%s" % acos_client.VERSION,
    }

    def __init__(self, host, port=None, protocol="https", timeout=None):
        if port is None:
            if protocol is 'http':
                port = 80
            else:
                port = 443
        self.url_base = "%s://%s:%s" % (protocol, host, port)

    def request(self, method, api_url, params={}, headers=None,
                file_name=None, file_content=None, axapi_args=None, **kwargs):
        LOG.debug("axapi_http: full url = %s", self.url_base + api_url)
        LOG.debug("axapi_http: %s url = %s", method, api_url)
        LOG.debug("axapi_http: params = %s", json.dumps(logutils.clean(params), indent=4))

        # Update params with axapi_args for currently unsupported configuration of objects
        if axapi_args is not None:
            formatted_axapi_args = dict([(k.replace('_', '-'), v) for k, v in
                                        axapi_args.iteritems()])
            params = acos_client.v21.axapi_http.merge_dicts(params, formatted_axapi_args)

        if (file_name is None and file_content is not None) or \
           (file_name is not None and file_content is None):
            raise ValueError("file_name and file_content must both be "
                             "populated if one is")

        hdrs = self.HEADERS.copy()
        if headers:
            hdrs.update(headers)

        if params:
            params_copy = params.copy()
            # params_copy.update(extra_params)
            LOG.debug("axapi_http: params_all = %s", logutils.clean(params_copy))

            payload = json.dumps(params_copy, encoding='utf-8')
        else:
            payload = None

        LOG.debug("axapi_http: headers = %s", json.dumps(logutils.clean(hdrs), indent=4))

        if file_name is not None:
            files = {
                'file': (file_name, file_content, "application/octet-stream"),
                'json': ('blob', payload, "application/json")
            }

            hdrs.pop("Content-type", None)
            hdrs.pop("Content-Type", None)
            z = requests.request(method, self.url_base + api_url, verify=False,
                                 files=files, headers=hdrs)
        else:
            z = requests.request(method, self.url_base + api_url, verify=False,
                                 data=payload, headers=hdrs)

        if z.status_code == 204:
            return None

        try:
            r = z.json()
        except ValueError as e:
            # The response is not JSON but it still succeeded.
            if z.status_code == 200:
                return {}
            else:
                raise e

        LOG.debug("axapi_http: data = %s", json.dumps(logutils.clean(r), indent=4))

        if 'response' in r and 'status' in r['response']:
            if r['response']['status'] == 'fail':
                    acos_responses.raise_axapi_ex(r, method, api_url)

        if 'authorizationschema' in r:
            acos_responses.raise_axapi_auth_error(
                r, method, api_url, headers)

        return r

    def get(self, api_url, params={}, headers=None, **kwargs):
        return self.request("GET", api_url, params, headers, **kwargs)

    def post(self, api_url, params={}, headers=None, **kwargs):
        return self.request("POST", api_url, params, headers, **kwargs)

    def put(self, api_url, params={}, headers=None, **kwargs):
        return self.request("PUT", api_url, params, headers, **kwargs)

    def delete(self, api_url, params={}, headers=None, **kwargs):
        return self.request("DELETE", api_url, params, headers, **kwargs)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import unittest2 as unittest

import acos_client.errors as acos_errors

import v21_mocks as mocks


class TestPartition(unittest.TestCase):

    # Test harness bug with delete
    # def test_partition_delete(self):
    #     with mocks.PartitionDelete().client() as c:
    #         c.system.partition.delete('p1')

    # def test_partition_delete_not_found(self):
    #     with mocks.PartitionDeleteNotFound().client() as c:
    #         c.system.partition.delete('p1')

    def test_partition_create(self):
        with mocks.PartitionCreate().client() as c:
            c.system.partition.create('p1')

    def test_partition_create_exists(self):
        with mocks.PartitionCreateExists().client() as c:
            with self.assertRaises(acos_errors.Exists):
                c.system.partition.create('p1')

    def test_partition_exists(self):
        with mocks.PartitionExists().client() as c:
            self.assertTrue(c.system.partition.exists('p1'))

    def test_partition_exists_not_found(self):
        with mocks.PartitionExistsNotFound().client() as c:
            self.assertFalse(c.system.partition.exists('p1'))

    def test_partition_active(self):
        with mocks.PartitionActive().client() as c:
            c.system.partition.active('p1')

    def test_partition_active_not_found(self):
        with mocks.PartitionActiveNotFound().client() as c:
            with self.assertRaises(acos_errors.NotFound):
                c.system.partition.active('p1')



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.errors as ae

import base


class Action(base.BaseV30):

    def write_memory(self, **kwargs):
        payload = {
            "memory": {
                "primary": True
            }
        }
        try:
            self._post("/write/memory/", payload, **kwargs)
        except ae.AxapiJsonFormatError:
            # Workaround regression in 4.1.0 backwards compat
            self._post("/write/memory/", "", **kwargs)

    def activate_and_write(self, partition, **kwargs):
        self.write_memory()



# Copyright (C) 2015, A10 Networks Inc. All rights reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import unittest
import uuid

from acos_client import errors as ae
from acos_client.v30 import responses as target


class TestResponse(unittest.TestCase):
    def _build_test_response(self, code, msg):
        return {
            "response": {
                "err": {
                    "code": code,
                    "msg": msg
                }
            }
        }

    def _test_raise_axapi_ex(self, response, method, api_url):
        self.assertRaises(ae.NotFound, target.raise_axapi_ex, response, method, api_url)

    def test_raise_axapi_ex_NotFound(self):
        not_found_codes = [1023443968, 1023475727, 1207959957, 520749062, 1023410176, 1023410181]
        # 1023410181 is a special case.  It matches on anything that's NOT DELETE and starts with
        # /axapi/v3/slb/service-group/.*/member/
        # That probably needs to be fixed and this test should break when it does
        test_obj = uuid.uuid4()
        test_url = "/axapi/v3/object/{0}".format(test_obj)
        test_method = "GET"
        test_msg = "Could not find object {0}".format(test_obj)

        for x in not_found_codes:
            test_response = self._build_test_response(x, test_msg)
            self._test_raise_axapi_ex(test_response, test_method, test_url)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
import unittest2 as unittest

from acos_client.v30.slb import service_group


class TestServiceGroup(unittest.TestCase):

    def test_sg_update_health_check_disable(self):
        client = mock.MagicMock()
        sg = service_group.ServiceGroup(client)

        sg.update('fake-pool-name', health_monitor="", health_check_disable=True)

        expected = {
            'service-group': {
                'name': 'fake-pool-name',
                'health-check-disable': 1
            }
        }

        ((method, url, params, header), kwargs) = client.http.request.call_args

        self.assertEqual(expected, params)

    def test_sg_update_health_check(self):
        client = mock.MagicMock()
        sg = service_group.ServiceGroup(client)

        sg.update('fake-pool-name', health_monitor="fake-hm")

        expected = {
            'service-group': {
                'name': 'fake-pool-name',
                'health-check': 'fake-hm'
            }
        }

        ((method, url, params, header), kwargs) = client.http.request.call_args

        self.assertEqual(expected, params)



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.v30.base as base

from hm import HealthMonitor
from server import Server
from service_group import ServiceGroup
from template import Template
from virtual_server import VirtualServer


class SLB(base.BaseV30):
    # For status args
    DOWN = 0
    UP = 1

    @property
    def hm(self):
        return HealthMonitor(self.client)

    @property
    def server(self):
        return Server(self.client)

    @property
    def service_group(self):
        return ServiceGroup(self.client)

    @property
    def template(self):
        return Template(self.client)

    @property
    def virtual_server(self):
        return VirtualServer(self.client)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.v21.base as base

from virtual_port import VirtualPort


class VirtualServer(base.BaseV21):

    @property
    def vport(self):
        return VirtualPort(self.client)

    def all(self, **kwargs):
        return self._get("slb.virtual_server.getAll", **kwargs)

    def get(self, name, **kwargs):
        return self._post("slb.virtual_server.search", {'name': name},
                          **kwargs)

    def _set(self, action, name, ip_address=None, status=1, **kwargs):
        params = {
            "virtual_server": self.minimal_dict({
                "name": name,
                "address": ip_address,
                "status": status,
            }),
        }

        self._post(action, params, **kwargs)

    def create(self, name, ip_address, status=1, **kwargs):
        self._set("slb.virtual_server.create", name, ip_address, status,
                  **kwargs)

    def update(self, name, ip_address=None, status=1, **kwargs):
        self._set("slb.virtual_server.update", name, ip_address, status,
                  **kwargs)

    def delete(self, name, **kwargs):
        self._post("slb.virtual_server.delete", {"name": name}, **kwargs)

    def stats(self, name, **kwargs):
        return self._post("slb.virtual_server.fetchStatistics", {"name": name},
                          **kwargs)

    def all_stats(self, **kwargs):
        return self._get("slb.virtual_server.fetchAllStatistics", **kwargs)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from acos_client import multipart
from action import Action
from admin import Admin
import base
from config_file import ConfigFile
from device_info import DeviceInfo
from log import Log
from partition import Partition


class System(base.BaseV21):
    def backup(self, **kwargs):
        return self._get("system.backup", **kwargs)

    def restore(self, name, data, **kwargs):
        m = multipart.Multipart()
        m.file(name="restore", filename=name, value=data)
        ct, payload = m.get()
        kwargs.update(payload=buffer(payload), headers={'Content-type': ct})
        return self._post("system.restore", **kwargs)

    def tech_download(self, **kwargs):
        return self._get("system.show_tech.download", **kwargs)

    def information(self):
        return self._get("system.information.get")

    def performance(self):
        return self._get("system.performance.get")

    @property
    def admin(self):
        return Admin(self.client)

    @property
    def device_info(self):
        return DeviceInfo(self.client)

    @property
    def action(self):
        return Action(self.client)

    @property
    def partition(self):
        return Partition(self.client)

    @property
    def config_file(self):
        return ConfigFile(self.client)

    @property
    def log(self):
        return Log(self.client)

    @property
    def banner(self):
        return self.Banner(self.client)

    class Banner(base.BaseV21):
        def get(self, **kwargs):
            return self._get('system.banner.get', **kwargs)

        def set(self, banner, **kwargs):
            params = {"banner": banner}
            return self._post('system.log.banner.set', params, **kwargs)

    @property
    def hostname(self):
        return self.Hostname(self.client)

    class Hostname(base.BaseV21):
        def get(self, **kwargs):
            return self._get('system.hostname.get', **kwargs)

        def set(self, hostname, **kwargs):
            params = {"hostname": hostname}
            return self._post('system.hostname.set', params, **kwargs)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.v21.base as base


class VirtualPort(base.BaseV21):

    # Protocols
    TCP = 2
    UDP = 3
    HTTP = 11
    HTTPS = 12
    OTHERS = 4
    RTSP = 5
    FTP = 6
    MMS = 7
    SIP = 8
    FAST_HTTP = 9
    GENERIC_PROXY = 10
    SSL_PROXY = 13
    SMTP = 14
    SIP_TCP = 15
    SIPS = 16
    DIAMETER = 17
    DNS_UDP = 18
    TFTP = 19
    DNS_TCP = 20
    RADIUS = 21
    MYSQL = 22
    MSSQL = 23
    FIX = 24
    SMPP_TCP = 25
    SPDY = 26
    SPDYS = 27
    FTP_PROXY = 28

    def _set(self, action, virtual_server_name, name, protocol, port,
             service_group_name,
             s_pers_name=None, c_pers_name=None, status=1, **kwargs):
        params = {
            "name": virtual_server_name,
            "vport": self.minimal_dict({
                "name": name,
                "service_group": service_group_name,
                "protocol": protocol,
                "port": int(port),
                "source_ip_persistence_template": s_pers_name,
                "cookie_persistence_template": c_pers_name,
                "status": status
            })
        }
        self._post(action, params, **kwargs)

    def create(self, virtual_server_name, name, protocol, port,
               service_group_name,
               s_pers_name=None, c_pers_name=None, status=1, **kwargs):
        self._set('slb.virtual_server.vport.create', virtual_server_name,
                  name, protocol, port, service_group_name,
                  s_pers_name, c_pers_name, status, **kwargs)

    def update(self, virtual_server_name, name, protocol, port,
               service_group_name,
               s_pers_name=None, c_pers_name=None, status=1, **kwargs):
            self._set('slb.virtual_server.vport.update', virtual_server_name,
                      name, protocol, port, service_group_name,
                      s_pers_name, c_pers_name, status, **kwargs)

    def delete(self, virtual_server_name, name, protocol, port, **kwargs):
        params = {
            "name": virtual_server_name,
            "vport": {
                "name": name,
                "protocol": protocol,
                "port": int(port)
            }
        }
        self._post("slb.virtual_server.vport.delete", params, **kwargs)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.v21.base as base


class BaseSSL(base.BaseV21):

    def get(self, name, **kwargs):
        return self._post(("slb.template.%s.search" % self.template_type),
                          {'name': name}, **kwargs)

    def _set(self, action, name, cert_name, key_name, **kwargs):
        params = {
            "%s_template" % self.template_type: {
                "cert_name": cert_name,
                "key_name": key_name,
                "name": name
            }
        }
        self._post(action, params, **kwargs)

    def create(self, name, cert_name, key_name, **kwargs):
        self._set(("slb.template.%s.create" % self.template_type),
                  name, cert_name, key_name, **kwargs)

    def update(self, name, cert_name, key_name, **kwargs):
        self._set(("slb.template.%s.update" % self.template_type),
                  name, cert_name, key_name, **kwargs)

    def delete(self, name, **kwargs):
        self._post(("slb.template.%s.delete" % self.template_type),
                   {"name": name}, **kwargs)


class ClientSSL(BaseSSL):

    template_type = "client_ssl"


class ServerSSL(BaseSSL):

    template_type = "server_ssl"



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import unittest2 as unittest

import acos_client.errors as acos_errors

import v21_mocks as mocks


class TestServiceGroup(unittest.TestCase):

    def test_sg_delete(self):
        with mocks.ServiceGroupDelete().client() as c:
            c.slb.service_group.delete('pool1')

    def test_sg_delete_not_found(self):
        with mocks.ServiceGroupDeleteNotFound().client() as c:
            c.slb.service_group.delete('pool1')

    def test_sg_create(self):
        with mocks.ServiceGroupCreate().client() as c:
            c.slb.service_group.create('pool1')

    def test_sg_create_exists(self):
        with mocks.ServiceGroupCreateExists().client() as c:
            with self.assertRaises(acos_errors.Exists):
                c.slb.service_group.create('pool1')

    def test_sg_update(self):
        with mocks.ServiceGroupUpdate().client() as c:
            c.slb.service_group.update(
                'pool1',
                lb_method=c.slb.service_group.LEAST_CONNECTION)

    def test_sg_update_not_found(self):
        with mocks.ServiceGroupUpdateNotFound().client() as c:
            with self.assertRaises(acos_errors.NotFound):
                c.slb.service_group.update(
                    'pool1',
                    lb_method=c.slb.service_group.LEAST_CONNECTION)

    def test_sg_search(self):
        with mocks.ServiceGroupSearch().client() as c:
            c.slb.service_group.get('pool1')

    def test_sg_search_not_found(self):
        with mocks.ServiceGroupSearchNotFound().client() as c:
            with self.assertRaises(acos_errors.NotFound):
                c.slb.service_group.get('pool1')



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.errors as acos_errors
import acos_client.v30.base as base

from virtual_port import VirtualPort


class VirtualServer(base.BaseV30):
    url_prefix = '/slb/virtual-server/'

    @property
    def vport(self):
        return VirtualPort(self.client)

    def all(self):
        return self._get(self.url_prefix)

    def get(self, name):
        return self._get(self.url_prefix + name)

    def _set(self, name, ip_address=None, status='stats-data-enable',
             update=False, **kwargs):
        params = {
            "virtual-server": self.minimal_dict({
                "name": name,
                "ip-address": ip_address,
            }),
        }

        if not update:
            name = ''

        self._post(self.url_prefix + name, params, **kwargs)

    def create(self, name, ip_address, status='stats-data-enable', **kwargs):
        try:
            self.get(name)
        except acos_errors.NotFound:
            pass
        else:
            raise acos_errors.Exists

        self._set(name, ip_address, status, **kwargs)

    def update(self, name, ip_address=None, status='stats-data-enable',
               **kwargs):
        self._set(name, ip_address, status, update=True, **kwargs)

    def delete(self, name):
        self._delete(self.url_prefix + name)

    def stats(self, name='', **kwargs):
        # resp = self._get(self.url_prefix + name + '/stats/')
        resp = self._get(self.url_prefix + name + '/stats', **kwargs)
        return resp
        # raise acos_errors.NotImplemented()



#!/usr/bin/env python
# flake8: noqa

from setuptools import setup, find_packages

setup(
    name = "acos-client",
    version = "1.2.9",
    packages = find_packages(),

    author = "A10 Networks",
    author_email = "dougw@a10networks.com",
    description = "A10 Networks ACOS API Client",
    license = "Apache",
    keywords = "a10 axapi acos adc slb load balancer",
    url = "https://github.com/a10networks/acos-client",

    long_description = open('README.md').read(),

    classifiers = [
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],

    install_requires = ['hash_ring>=1.3.1', 'requests>=2.3.0']
)



# Copyright 2016, All Rights Reserved,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import unittest2

from acos_client import logutils as target


class TestLogutils(unittest2.TestCase):
    def setUp(self):
        self.clean_fields = ["username", "password"]

        self.obj_flat = self._fakeobj(a=1, b=2, username="admin", password="secret")
        self.obj_onelevel = self._fakeobj(a=1,
                                          b=2,
                                          credentials=self._fakeobj(
                                              username="admin",
                                              password="secret"))
        self.obj_twolevel = self._fakeobj(a=1,
                                          b=2,
                                          credentials=self._fakeobj(
                                              inside_secret=self._fakeobj(
                                                  username="admin",
                                                  password="secret")))
        self.flat_dict = {
            "a": 1,
            "b": 2,
            "username": "admin",
            "password": "secret"
        }

        self.dict_onelevel = {
            "a": 1,
            "b": 2,
            "credentials": {
                "username": "admin",
                "password": "secret"
            }
        }

        self.dict_twolevel = {
            "a": 1,
            "b": 2,
            "credentials": {
                "inside_secret": {
                    "username": "admin",
                    "password": "secret"
                }
            }
        }

    def _fakeobj(self, *args, **kwargs):
        return FakeObject(**kwargs)

    def test_clean_flat_dict(self):
        actual = target.clean(self.flat_dict)
        self.assertEqual(target.REPLACEMENT, actual["username"])
        self.assertEqual(target.REPLACEMENT, actual["password"])
        self.assertEqual(self.flat_dict["a"], actual["a"])

    def test_clean_onelevel_dict(self):
        actual = target.clean(self.dict_onelevel).get("credentials")

        self.assertEqual(target.REPLACEMENT, actual["username"])
        self.assertEqual(target.REPLACEMENT, actual["password"])
        self.assertEqual(1, self.dict_onelevel["a"])

    def test_clean_twolevel_dict(self):
        actual = target.clean(self.dict_twolevel).get("credentials").get("inside_secret")

        self.assertEqual(target.REPLACEMENT, actual["username"])
        self.assertEqual(target.REPLACEMENT, actual["password"])

    def test_obj_flat(self):
        actual = target.clean(self.obj_flat)
        self.assertEqual(target.REPLACEMENT, actual.username)
        self.assertEqual(target.REPLACEMENT, actual.password)

    def test_obj_onelevel(self):
        actual = target.clean(self.obj_onelevel).credentials
        self.assertEqual(target.REPLACEMENT, actual.username)
        self.assertEqual(target.REPLACEMENT, actual.password)

    def test_obj_twolevel(self):
        actual = target.clean(self.obj_twolevel).credentials.inside_secret
        self.assertEqual(target.REPLACEMENT, actual.username)
        self.assertEqual(target.REPLACEMENT, actual.password)


class FakeObject(object):
    def __init__(self, *args, **kwargs):
        for k, v in kwargs.iteritems():
            setattr(self, k, v)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import base


class Log(base.BaseV21):

        def set(self, sys_log, **kwargs):
            params = {"sys_log": sys_log}
            return self._post("system.log.set", params, **kwargs)

        def get(self, **kwargs):
            return self._get("system.log.get", **kwargs)

        def clear(self, sys_log, **kwargs):
            return self._post("system.log.clear", **kwargs)

        def download(self, **kwargs):
            return self._get('system.log.download', **kwargs)

        def backup(self, **kwargs):
            return self._post('system.log.backup', **kwargs)

        @property
        def level(self):
            return self.Level(self.client)

        @property
        def server(self):
            return self.Server(self.client)

        @property
        def buffer(self):
            return self.Buffer(self.client)

        @property
        def smtp(self):
            return self.Smtp(self.client)

        @property
        def audit(self):
            return self.Audit(self.client)

        class Level(base.BaseV21):
            def get(self, **kwargs):
                return self._get('system.log.level.get', **kwargs)

            def set(self, log_level, **kwargs):
                params = {"log_level": log_level}
                return self._post('system.log.level.set', params, **kwargs)

        class Server(base.BaseV21):
            def get(self, **kwargs):
                return self._get('system.log.server.get', **kwargs)

            def set(self, log_server, **kwargs):
                params = {"log_server": log_server}
                return self._post('system.log.server.set', params, **kwargs)

        class Buffer(base.BaseV21):
            def get(self, **kwargs):
                return self._get('system.log.buffer.get', **kwargs)

            def set(self, buff_size, **kwargs):
                params = {"buffer_size": buff_size}
                return self._post('system.log.buffer.set', params, **kwargs)

        class Smtp(base.BaseV21):
            def get(self, **kwargs):
                return self._get('system.log.smtp.get', **kwargs)

            def set(self, smtp, **kwargs):
                params = {"smtp": smtp}
                return self._post('system.log.smtp.set', params, **kwargs)

        class Audit(base.BaseV21):
            def get(self, **kwargs):
                return self._get('system.log.audit.get', **kwargs)

            def set(self, audit, **kwargs):
                params = {"audit": audit}
                return self._post('system.log.audit.set', params, **kwargs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import re

from acos_client import multipart
import acos_client.v21.base as base


class ClassList(base.BaseV21):

    @staticmethod
    def _fix_json(data):
        p = re.compile(r'(?<=[^:{\[,])"(?![:,}\]])')
        return json.loads(re.sub(p, '\\"', data))

    def all(self, **kwargs):
        return self._fix_json(self._get("slb.class_list.getAll", **kwargs))

    def get(self, name, **kwargs):
        return ClassList._fix_json(self._post("slb.class_list.search",
                                              {'name': name}, **kwargs))

    def download(self, name, **kwargs):
        return self._post('slb.class_list.download',
                          params={'file_name': name}, **kwargs)

    def upload(self, name, class_list, **kwargs):
        m = multipart.Multipart()
        m.file(name=name, filename=name, value=class_list)
        ct, payload = m.get()
        kwargs.update(payload=payload, headers={'Content-type': ct})
        return self._post('slb.class_list.upload', **kwargs)

    def _set(self, action, class_list, **kwargs):
        return self._post(action, class_list, **kwargs)

    def create(self, class_list, **kwargs):
        return self._set("slb.class_list.create", class_list, **kwargs)

    def update(self, class_list, **kwargs):
        return self._set("slb.class_list.update", class_list, **kwargs)

    def delete(self, name, **kwargs):
        self._post("slb.class_list.delete", {"name": name}, **kwargs)



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class BaseV30(object):

    def __init__(self, client):
        self.client = client
        self.http = client.http
        self.auth_header = {}

    def minimal_dict(self, my_dict, exclude=[]):
        return dict((k, v) for k, v in my_dict.items() if v is not None or k in exclude)

    def url(self, action):
        self.auth_header['Authorization'] = "A10 %s" % self.client.session.id
        return ("/axapi/v3" + action)

    def _request(self, method, action, params, retry_count=0, **kwargs):
        return self.client.http.request(method, self.url(action), params,
                                        self.auth_header, **kwargs)

    def _get(self, action, params={}, **kwargs):
        return self._request('GET', action, params, **kwargs)

    def _post(self, action, params={}, **kwargs):
        return self._request('POST', action, params, **kwargs)

    def _delete(self, action, params={}, **kwargs):
        return self._request('DELETE', action, params, **kwargs)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

import mock
import unittest

import acos_client

DEFAULT_SESSION_ID = 'session0'


class MockPairClient(object):

    def __init__(self, parent, session_id=None):
        self.parent = parent
        self.session_id = session_id

    def __enter__(self):
        c = acos_client.Client('localhost', acos_client.AXAPI_21,
                               self.parent.username,
                               self.parent.password)
        c.http._http = self.parent.mock()
        c.session.http._http = c.http._http
        if self.session_id is not None:
            c.session.session_id = self.session_id

        return c

    def __exit__(self, *args, **kwargs):
        self.parent.post_validate()


class MockPair(unittest.TestCase):
    method = 'POST'
    action = None
    params = None
    response = None

    def __init__(self, fields={}):
        self.testMethodPrefix = "DONTRUNME"
        self.fields = fields
        self.session_id = fields.get('session_id', 'session0')
        self.username = fields.get('username', 'defuser')
        self.password = fields.get('password', 'defpass')

    def url(self, session_id=None):
        return ("/services/rest/v2.1/?format=json&method=%s&session_id=%s" %
                (self.action, session_id))

    def client(self, session_id=None):
        self._client = MockPairClient(self, session_id=session_id)
        return self._client

    def mock(self):
        self._mock = mock.MagicMock(return_value=json.dumps(self.output()))
        return self._mock

    def output(self):
        if self.response is not None:
            return self.response
        else:
            return

    def post_validate(self):
        if self.action is not None:
            validated = False
            if self.params is not None:
                for name, args, kwargs in self._mock.mock_calls:
                    if args and len(args) > 2 and args[2].__class__ == str:
                        if json.loads(args[2]) == self.params:
                            validated = True
                    else:
                        validated = True
            else:
                validated = True
            self.assertTrue(validated)


class AuthenticatedMockPair(MockPair):
    response = {"response": {"status": "OK"}}

    def client(self):
        return super(AuthenticatedMockPair, self).client(
            session_id=self.session_id)


class Session(MockPair):

    def output(self):
        return {'session_id': self.session_id}

    def post_validate(self):
        self._mock.assert_called_once_with(
            'POST',
            '/services/rest/v2.1/?format=json&method=authenticate',
            json.dumps({'username': self.username, 'password': self.password}))


class SessionBadPassword(Session):

    def output(self):
        return {
            "response": {
                "status": "fail",
                "err": {"code": 520486915, "msg": " Admin password error"}
            }
        }


class Close(AuthenticatedMockPair):
    action = 'session.close'
    params = {'session_id': DEFAULT_SESSION_ID}


class CloseBadSession(Close):
    params = {'session_id': "badsessionid"}
    response = {
        "response": {
            "status": "fail",
            "err": {"code": 1009, "msg": "Invalid session ID"}
        }
    }


class SystemInformation(AuthenticatedMockPair):
    method = 'GET'
    action = 'system.information.get'
    params = {}
    response = {
        'system_information': {
            'advanced_core_os_on_compact_flash1': 'No Software',
            'advanced_core_os_on_compact_flash2': 'No Software',
            'advanced_core_os_on_harddisk1': '2.7.1-P3-AWS(build: 4)',
            'advanced_core_os_on_harddisk2': '2.7.1-P3-AWS(build: 4)',
            'aflex_engine_version': '2.0.0',
            'axapi_version': '2.1',
            'current_time': '03:25:47 IST Tue Jul 1 2014',
            'firmware_version': 'N/A',
            'last_config_saved': '06:25:26 GMT Sat Dec 28 2013',
            'serial_number': 'N/A',
            'software_version': '2.7.1-P3-AWS(build: 4)',
            'startup_mode': 'hard disk primary',
            'technical_support': 'www.a10networks.com/support '
        }
    }


class SystemWriteMemory(AuthenticatedMockPair):
    method = 'GET'
    action = 'system.action.write_memory'
    params = {}


class Server(AuthenticatedMockPair):
    params = {'name': 's1'}


class ServerDelete(Server):
    action = 'slb.server.delete'
    params = {"server": {"name": 's1'}}


class ServerDeleteNotFound(ServerDelete):
    response = {
        "response": {
            "status": "fail",
            "err": {
                "code": 67174402,
                "msg": " No such Server"}
        }
    }


class ServerCreate(Server):
    action = 'slb.server.create'
    params = {'server': {'host': '192.168.2.254', 'name': 's1'}}


class ServerCreateExists(ServerCreate):
    response = {"response": {"status": "fail", "err": {"code": 402653200,
                "msg": " Name or IP address already exists."}}}


class ServerSearch(Server):
    action = 'slb.server.search'
    response = {
        'server': {
            'status': 1, 'conn_resume': 0, 'weight': 1,
            'conn_limit': 8000000, 'host': '192.168.2.254',
            'spoofing_cache': 0, 'port_list': [],
            'gslb_external_address': '0.0.0.0', 'slow_start': 0,
            'name': 's1', 'health_monitor': '(default)',
            'extended_stats': 0, 'template': 'default', 'stats_data': 1,
            'conn_limit_log': 0
        }
    }


class ServerSearchNotFound(ServerSearch):
    response = {"response": {"status": "fail", "err": {"code": 67174402,
                "msg": " No such Server"}}}


class ServiceGroup(AuthenticatedMockPair):
    params = {'name': 'pool1'}


class ServiceGroupDelete(ServiceGroup):
    action = 'slb.service_group.delete'


class ServiceGroupDeleteNotFound(ServiceGroupDelete):
    response = {"response": {"status": "fail", "err": {"code": 67305473,
                "msg": " No such service group"}}}


class ServiceGroupCreate(ServiceGroup):
    action = 'slb.service_group.create'
    params = {'service_group': {'lb_method': 0, 'protocol': 2,
              'name': 'pool1'}}


class ServiceGroupCreateExists(ServiceGroupCreate):
    response = {"response": {"status": "fail", "err": {"code": 402653201,
                "msg": " Service group already exists."}}}


class ServiceGroupSearch(ServiceGroup):
    action = 'slb.service_group.search'
    response = {
        'service_group': {
            'lb_method': 0,
            'protocol': 2,
            'name': 'pool1',
            'backup_server_event_log_enable': 0,
            'health_monitor': '',
            'client_reset': 0,
            'min_active_member': {
                'status': 0,
                'number': 0,
                'priority_set': 0
            },
            'extended_stats': 0,
            'stats_data': 1,
            'member_list': []
        }
    }


class ServiceGroupSearchNotFound(ServiceGroupSearch):
    response = {"response": {"status": "fail", "err": {"code": 67305473,
                "msg": " No such service group"}}}


class ServiceGroupUpdate(ServiceGroup):
    action = 'slb.service_group.update'
    params = {"service_group": {"lb_method": 2, "name": "pool1"}}


class ServiceGroupUpdateNotFound(ServiceGroupUpdate):
    response = {"response": {"status": "fail", "err": {"code": 67305473,
                "msg": " No such service group"}}}


class VirtualServer(AuthenticatedMockPair):
    params = {'name': 'vip1'}


class VirtualServerDelete(VirtualServer):
    action = 'slb.virtual_server.delete'


class VirtualServerDeleteNotFound(VirtualServerDelete):
    response = {"response": {"status": "fail", "err": {"code": 67239937,
                "msg": " No such Virtual Server"}}}


class VirtualServerCreate(VirtualServer):
    action = 'slb.virtual_server.create'
    params = {'virtual_server': {'status': 1, 'name': 'vip1',
              'address': '192.168.2.250'}}


class VirtualServerCreateExists(VirtualServerCreate):
    response = {"response": {"status": "fail", "err": {"code": 402653206,
                "msg": " Name already exists."}}}


class VirtualServerSearch(VirtualServer):
    action = 'slb.virtual_server.search'
    response = {
        "virtual_server": {
            "name": "vip1",
            "address": "192.168.2.250",
            "status": 1,
            "vrid": 0,
            "arp_status": 1,
            "stats_data": 1,
            "extended_stats": 0,
            "disable_vserver_on_condition": 0,
            "redistribution_flagged": 0,
            "ha_group": {
                "status": 0,
                "ha_group_id": 0,
                "dynamic_server_weight": 0
            },
            "vip_template": "shared/default",
            "pbslb_template": "",
            "vport_list": []
        }
    }


class VirtualServerSearchNotFound(VirtualServerSearch):
    response = {"response": {"status": "fail", "err": {"code": 67239937,
                "msg": " No such Virtual Server"}}}


class VirtualPort(AuthenticatedMockPair):
    params = {
        'vport': {
            'protocol': 11,
            'name': 'vip1_VPORT',
            'port': 80
        },
        'name': 'vip1'
    }


class VirtualPortDelete(VirtualPort):
    action = 'slb.virtual_server.vport.delete'


class VirtualPortDeleteNotFound(VirtualPortDelete):
    response = {"response": {"status": "fail", "err": {"code": 1043,
                "msg": "Can not find the virtual server port"}}}


class VirtualPortCreate(VirtualPort):
    action = 'slb.virtual_server.vport.create'
    params = {
        'vport': {
            'service_group': 'pool1',
            'status': 1,
            'protocol': 11,
            'name': 'vip1_VPORT',
            'port': 80
        },
        'name': 'vip1'
    }


class VirtualPortCreateExists(VirtualPortCreate):
    response = {"response": {"status": "fail", "err": {"code": 1406,
                "msg": "The virtual port already exists."}}}


class HealthMonitor(AuthenticatedMockPair):
    params = {'name': 'hm1'}


class HealthMonitorDelete(HealthMonitor):
    action = 'slb.hm.delete'


class HealthMonitorDeleteNotFound(HealthMonitorDelete):
    response = {"response": {"status": "fail", "err": {"code": 33619968,
                "msg": " The monitor does not exist."}}}


class HealthMonitorCreate(HealthMonitor):
    action = 'slb.hm.create'
    params = {"retry": 5, "name": "hm1", "consec_pass_reqd": 5, "interval": 5,
              "timeout": 5, "disable_after_down": 0, "type": "HTTP"}


class HealthMonitorCreateExists(HealthMonitorCreate):
    response = {"response": {"status": "fail", "err": {"code": 2941,
                "msg": "The same health monitor name already exist."}}}


class HealthMonitorSearch(HealthMonitor):
    action = 'slb.hm.search'
    response = {
        "health_monitor": {
            "name": "hfoobar",
            "retry": 5,
            "consec_pass_reqd": 5,
            "interval": 5,
            "timeout": 5,
            "strictly_retry": 0,
            "disable_after_down": 0,
            "override_ipv4": "0.0.0.0",
            "override_ipv6": "::",
            "override_port": 0,
            "type": 3,
            "http": {
                "port": 80,
                "host": "",
                "url": "GET /",
                "user": "",
                "password": "",
                "expect_code": "200",
                "maintenance_code": ""
            }
        }
    }


class HealthMonitorSearchNotFound(HealthMonitorSearch):
    response = {"response": {"status": "fail", "err": {"code": 33619968,
                "msg": " The monitor does not exist."}}}


class HealthMonitorUpdate(HealthMonitor):
    action = 'slb.hm.update'
    params = {"retry": 5, "name": "hm1", "consec_pass_reqd": 5, "interval": 5,
              "timeout": 5, "disable_after_down": 0, "type": "HTTP"}


class HealthMonitorUpdateNotFound(HealthMonitorUpdate):
    response = {"response": {"status": "fail", "err": {"code": 33619968,
                "msg": " The monitor does not exist."}}}


class Member(AuthenticatedMockPair):
    params = {'member': {'port': 80, 'server': 's1'}, 'name': 'pool1'}


class MemberDelete(Member):
    action = 'slb.service_group.member.delete'


class MemberDeleteNotFound(MemberDelete):
    response = {"response": {"status": "fail", "err": {"code": 1023,
                "msg": "Can not find the service group member"}}}


class MemberCreate(Member):
    action = 'slb.service_group.member.create'
    params = {'member': {'status': 1, 'port': 80, 'server': 's1'},
              'name': 'pool1'}


class MemberCreateExists(MemberCreate):
    response = {"response": {"status": "fail", "err": {"code": 1405,
                "msg": "The service group member already exists."}}}


class MemberUpdate(Member):
    action = 'slb.service_group.member.update'
    params = {'member': {'status': 0, 'port': 80, 'server': 's1'},
              'name': 'pool1'}


class MemberUpdateNotFound(MemberUpdate):
    response = {"response": {"status": "fail", "err": {"code": 1023,
                "msg": "Can not find the service group member"}}}


class MemberUpdateNoSuchServiceGroup(MemberUpdate):
    response = {"response": {"status": "fail", "err": {"code": 67305473,
                "msg": " No such service group"}}}


class SourceIpPersistence(AuthenticatedMockPair):
    params = {'name': 'sip1'}


class SourceIpPersistenceDelete(SourceIpPersistence):
    action = 'slb.template.src_ip_persistence.delete'


class SourceIpPersistenceDeleteNotFound(SourceIpPersistenceDelete):
    response = {"response": {"status": "fail", "err": {"code": 67371009,
                "msg": " No such Template"}}}


class SourceIpPersistenceCreate(SourceIpPersistence):
    action = 'slb.template.src_ip_persistence.create'
    params = {'src_ip_persistence_template': {'name': 'sip1'}}


class SourceIpPersistenceCreateExists(SourceIpPersistenceCreate):
    response = {"response": {"status": "fail", "err": {"code": 402653202,
                "msg": " Template name already exists."}}}


class SourceIpPersistenceSearch(SourceIpPersistence):
    action = 'slb.template.src_ip_persistence.search'
    response = {
        "src_ip_persistence_template": {
            "name": "sip1",
            "match_type": 0,
            "match_all": 0,
            "timeout": 5,
            "no_honor_conn": 0,
            "incl_sport": 0,
            "include_dstip": 0,
            "hash_persist": 0,
            "enforce_high_priority": 0,
            "netmask": "255.255.255.255",
            "netmask6": 128
        }
    }


class SourceIpPersistenceSearchNotFound(SourceIpPersistenceSearch):
    response = {"response": {"status": "fail", "err": {"code": 67371009,
                "msg": " No such Template"}}}


class HttpCookiePersistence(AuthenticatedMockPair):
    params = {'name': 'cp1'}


class HttpCookiePersistenceDelete(HttpCookiePersistence):
    action = 'slb.template.cookie_persistence.delete'


class HttpCookiePersistenceDeleteNotFound(HttpCookiePersistenceDelete):
    response = {"response": {"status": "fail", "err": {"code": 67371009,
                "msg": " No such Template"}}}


class HttpCookiePersistenceCreate(HttpCookiePersistence):
    action = 'slb.template.cookie_persistence.create'
    params = {'cookie_persistence_template': {'name': 'cp1'}}


class HttpCookiePersistenceCreateExists(HttpCookiePersistenceCreate):
    response = {"response": {"status": "fail", "err": {"code": 402653202,
                "msg": " Template name already exists."}}}


class HttpCookiePersistenceSearch(HttpCookiePersistence):
    action = 'slb.template.cookie_persistence.search'
    response = {
        "cookie_persistence_template": {
            "name": "cp1",
            "expire_exist": 0,
            "expire": 0,
            "cookie_name": "",
            "domain": "",
            "path": "",
            "match_type": 0,
            "match_all": 0,
            "insert_always": 0,
            "dont_honor_conn": 0
        }
    }


class HttpCookiePersistenceSearchNotFound(HttpCookiePersistenceSearch):
    response = {"response": {"status": "fail", "err": {"code": 67371009,
                "msg": " No such Template"}}}


class Partition(AuthenticatedMockPair):
    params = {'name': 'p1'}


class PartitionExists(Partition):
    action = 'system.partition.search'
    response = {
        "partition": {
            "partition_id": 1,
            "name": "p1",
            "max_aflex_file": 32,
            "network_partition": 0
        }
    }


class PartitionExistsNotFound(PartitionExists):
    response = {"response": {"status": "fail", "err": {"code": 520749062,
                "msg": " Partition does not exist."}}}


class PartitionActive(Partition):
    action = 'system.partition.active'


class PartitionActiveNotFound(PartitionActive):
    response = {"response": {"status": "fail", "err": {"code": 402718800,
                "msg": " Failed to get partition."}}}


class PartitionCreate(Partition):
    action = 'system.partition.create'
    params = {'partition': {'max_aflex_file': 32, 'network_partition': 0,
              'name': 'p1'}}


class PartitionCreateExists(PartitionCreate):
    response = {"response": {"status": "fail", "err": {"code": 1982,
                "msg": "The partition already exists"}}}


class PartitionDelete(Partition):
    action = 'system.partition.delete'


class PartitionDeleteNotFound(PartitionDelete):
    response = {"response": {"status": "fail", "err": {"code": 520749062,
                "msg": " Partition does not exist."}}}


class HASync(AuthenticatedMockPair):
    action = 'ha.sync_config'
    params = {
        'ha_config_sync': {
            'peer_operation': 0,
            'destination_ip': '192.168.2.254',
            'peer_reload': 0,
            'user': 'admin',
            'auto_authentication': 0,
            'sync_all_partition': 1,
            'operation': 2,
            'password': 'a10'
        }
    }



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import unittest2 as unittest

import acos_client.errors as acos_errors

import v21_mocks as mocks


class TestHttpCookiePersistence(unittest.TestCase):

    def test_http_cookie_delete(self):
        with mocks.HttpCookiePersistenceDelete().client() as c:
            c.slb.template.cookie_persistence.delete('cp1')

    def test_http_cookie_delete_not_found(self):
        with mocks.HttpCookiePersistenceDeleteNotFound().client() as c:
            c.slb.template.cookie_persistence.delete('cp1')

    def test_http_cookie_create(self):
        with mocks.HttpCookiePersistenceCreate().client() as c:
            c.slb.template.cookie_persistence.create('cp1')

    def test_http_cookie_create_exists(self):
        with mocks.HttpCookiePersistenceCreateExists().client() as c:
            with self.assertRaises(acos_errors.Exists):
                c.slb.template.cookie_persistence.create('cp1')

    def test_http_cookie_search(self):
        with mocks.HttpCookiePersistenceSearch().client() as c:
            c.slb.template.cookie_persistence.get('cp1')

    def test_http_cookie_search_not_found(self):
        with mocks.HttpCookiePersistenceSearchNotFound().client() as c:
            with self.assertRaises(acos_errors.NotFound):
                c.slb.template.cookie_persistence.get('cp1')



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import unittest2 as unittest

import acos_client

servers = ['a', 'b', 'c', 'd', 'e', 'f']


class TestHash(unittest.TestCase):

    def test_hash(self):
        h = acos_client.Hash(servers)
        a = h.get_server('aaa')
        h.get_server('bbb')
        h.get_server('ccc')
        self.assertEqual(a, h.get_server('aaa'))
